package nl.kallestruik.darena.util

import nl.kallestruik.darena.exceptions.MaterialNotFoundException
import nl.kallestruik.darena.types.ConfigSaveable
import nl.kallestruik.darena.types.ConfigLoadable
import nl.kallestruik.darena.types.ConfigListSaveable
import nl.kallestruik.darena.types.ConfigListLoadable
import org.bukkit.Material
import org.bukkit.configuration.file.YamlConfiguration
import org.bukkit.configuration.ConfigurationSection
import java.io.*

object ConfigHelper {

    fun getOrCreateConfig(file: File, template: String): YamlConfiguration {
        Logger.trace(ConfigHelper::class, "getOrCreateConfig(file: ${file.toString()}, template: $template)")
        if (!file.parentFile.exists())
            file.parentFile.mkdirs()
        if (!file.exists())
            createConfig(file, template)

        val config = YamlConfiguration()
        config.load(file)
        return config
    }

    private fun createConfig(file: File, template: String) {
        Logger.trace(ConfigHelper::class, "createConfig(file: ${file.toString()}, template: $template)")
        try {
            ConfigHelper.javaClass.getResourceAsStream("/$template").use { stream ->
                FileOutputStream(file).use { resStreamOut ->
                    if (stream == null) {
                        Logger.error(ConfigHelper::class, "Cannot get resource \"$template\" from jar file.")
                        throw IOException("Cannot get resource \"$template\" from Jar file.")
                    }
                    var readBytes: Int
                    val buffer = ByteArray(4096)
                    while (stream.read(buffer).also { readBytes = it } > 0) {
                        resStreamOut.write(buffer, 0, readBytes)
                    }
                }
            }
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }

    fun getOrCreateSection(section: ConfigurationSection, key: String): ConfigurationSection {
        Logger.trace(ConfigHelper::class, "getOrCreateSection(section: ${section.currentPath}, key: $key)")
        if (section.contains(key) && section.isConfigurationSection(key)) {
            return section.getConfigurationSection(key)!!
        }

        return section.createSection(key)
    }

    fun <T: ConfigSaveable> saveMap(section: ConfigurationSection, map: Map<String, T>) {
        Logger.trace(ConfigHelper::class, "saveMap(section: ${section.currentPath}, map: ${map.toString()})")
        for (entry in map.entries) {
            entry.value.save(ConfigHelper.getOrCreateSection(section, entry.key))
        }
    }

    fun <T: ConfigListSaveable> saveList(section: ConfigurationSection, list: List<T>) {
        Logger.trace(ConfigHelper::class, "saveList(section: ${section.currentPath}, list: ${list.toString()})")
        for (item in list) {
            section.set(item.getKey(), item.getValue())
        }
    }

    fun <R, T: ConfigLoadable<R>> loadMap(section: ConfigurationSection, loader: T): MutableMap<String, R> {
        Logger.trace(ConfigHelper::class, "loadMap(section: ${section.currentPath}, loader: ${loader::class.qualifiedName})")
        val map = mutableMapOf<String, R>()
        for (key in section.getKeys(false)) {
            map.put(key, loader.load(section.getConfigurationSection(key)!!))
        }

        return map
    }

    fun <R, T: ConfigListLoadable<R>> loadList(section: ConfigurationSection, loader: T): MutableList<R> {
        Logger.trace(ConfigHelper::class, "loadList(section: ${section.currentPath}, loader: ${loader::class.qualifiedName})")
        val list = mutableListOf<R>()
        for (key in section.getKeys(false)) {
            loader.loadFromList(key, section.get(key)!!)
        }

        return list
    }

    @Throws(MaterialNotFoundException::class)
    fun matchMaterial(material: String): Material {
        Logger.trace(ConfigHelper::class, "matchMaterial(material: $material)")
        return Material.matchMaterial(material) ?: throw MaterialNotFoundException("There is no material with the name '$material'")
    }

    fun parseIntList(string: String): List<Int> {
        Logger.trace(ConfigHelper::class, "parseIntList(string: $string)")
        val list = mutableListOf<Int>()
        for (key in string.split(" ")) {
            list.add(key.toInt())
        }

        return list
    }

    fun intListToString(list: List<Int>): String {
        Logger.trace(ConfigHelper::class, "list: ${list.toString()}")
        val sb = StringBuilder()
        for (i in list) {
            sb.append(i)
        }

        return sb.toString().trim()
    }

    fun parseStringIntListMap(section: ConfigurationSection): Map<String, List<Int>> {
        Logger.trace(ConfigHelper::class, "parseStringIntListMap(section: ${section.currentPath})")
        val map = mutableMapOf<String, List<Int>>()
        for (key in section.getKeys(true)) {
            map[key] = parseIntList(section.getString(key)!!)
        }

        return map
    }
}
